@@ -23,6 +23,9 @@ has 'dbh' =>
       default   => \&_make_dbh,
     );
 
+after '_unset_dbh' =>
+    sub { $_[0]->_clear_supports_nested_transactions() };
+
 has 'dsn' =>
     ( is        => 'rw',
       isa       => 'Str',
@@ -61,6 +64,14 @@ has 'auto_refresh' =>
       default => 1,
     );
 
+has 'allows_nested_transactions' =>
+    ( is      => 'ro',
+      isa     => 'Bool',
+      lazy    => 1,
+      default => \&_check_nested_transactions,
+      clearer => '_clear_supports_nested_transactions',
+    );
+
 has '_threaded' =>
     ( is       => 'ro',
       isa      => 'Bool',
@@ -143,6 +154,37 @@ sub _make_dbh
     return $self->_dbh();
 }
 
+sub _check_nested_transactions
+{
+    my $self = shift;
+
+    my $dbh = $self->dbh();
+
+    my $allows_nested =
+        eval
+        {
+            # This error comes from DBI in its default implementation
+            # of begin_work(). There didn't seem to be a way to shut
+            # this off (PrintWarn does not help). Hopefully the
+            # message text won't change.
+            local $SIG{__WARN__}
+                = sub { warn @_ unless $_[0] =~ /Already in a transaction/ };
+
+            $dbh->begin_work();
+            $dbh->begin_work();
+            $dbh->rollback();
+            $dbh->rollback();
+            1;
+        };
+
+    if ($@)
+    {
+        $dbh->rollback() unless $dbh->{AutoCommit};
+    }
+
+    return $allows_nested;
+}
+
 sub _set_pid_tid
 {
     my $self = shift;
@@ -288,6 +330,13 @@ This method reutrns attributes hash reference for the source. This
 will be a combination of any attributes passed to the constructor plus
 the L<REQUIRED ATTRIBUTES>.
 
+=head2 $source->allows_nested_transactions()
+
+Returns a boolean indicating whether or not the database to which the
+source connects supports nested transactions. It does this by trying
+to issue two calls to C<< $dbh->begin_work() >> followed by two calls
+to C<< $dbh->rollback() >> (in an eval block).
+
 =head1 REQUIRED ATTRIBUTES
 
 In order to provide consistency for C<Fey::ORM>, sources enforce a set
